function smoothness = D1CriticalExponentV2(mask_a, supp_a, dilation_d, sumrule_k)
%#####################Copyright and Notice###################################
%Copyright since 2006, ZHUANG Xiaosheng, University of Alberta,
%                    All Rights Reserved 
%Report errors, mistakes, and bugs to ZHUANG Xiaosheng at xzhuang@math.ualberta.ca.
%Send comments/suggestions to ZHUANG Xiaosheng at xzhuang@math.ualberta.ca.
%
%  This program is based on the following paper (Theorem 7.1):
%  Bin Han, "Vector cascade algorithms and refinable function vectors in
%  Sobolev spaces", Journal of Approximation Theory, Volumn 124 (2003), 
%  Issue 1, 44-88. 
%
%  1. Refinement Equation(in Tex form)
%  \Phi(x) = [\Phi_1(x) ... \Phi_r(x)]^T
%  \Phi(x) = |detM|\sum_{\beta in Z^s}{a(\beta)*\Phi(Mx-\beta)} 
%
%  2. Parameters
%  dimension: s = 1
%  multiplicity: r integer
%  #{support Mask_A}: N integer
%  Example: s=2, r=2, N=3
%
%
%  3. Input <=
%     3.1 mask_a is r*(rN) matrix, can be seen as N blocks, each block is
%         a r*r matrix. Example:
%        mask_a = [ 1 1  2 2  3 3;
%                   1 1  2 2  3 3 ] has 3 blocks of 2*2 matrices.
%     3.2 supp_a is the support of the mask_a. It is an s*N matrix.
%         supp_a = [\beta_1 ... \beta_N]
%         \beta_1, .. , \beta_N are s*1 integer vectors in normal order.
%         i.e., \beta_1 <= beta_2 iff. there exists j, s.t. \beta_{1j}<=beta_{2j}
%         Example: s=1, N=3
%         supp_a = [ -1 0 1 3];
%         Note: mask_a(\beta_j) = the j_th block of Mask_A
%     3.3 dilation_d is an integer larger than 1.
%     3.4 sumrule_k. mask_a satisfies the sum rules of oder k with
%     respect to some vector \hat{y}(\xi).
%  4. Output =>
%     smoothness: the estimate of the L_2 critical exponent. see the
%     definition in the above paper.
% 
%  NOTE: Here we don't check the sum rule of the mask a. We assume that
%  mask a is obtained previously and already satisfied the sum rule of
%  order at leat 1.
%
% Last modified:  July 10, 2006%

if nargin~=4 error('this routines requies 4 input argument'); end

%Check the arguments.
r = size(mask_a,1);
s = size(supp_a,1);
d= abs(dilation_d);
suppN = size(supp_a,2);

eps = 1e-7;
if suppN ~= size(mask_a,2)/r error('Mask a and its support are not consistent!'); end
if d <= 1 error('Dilation d must be larger than 1!'); end
% EV_M = eig(M);
% for i=1:length(EV_M)-1
%     for j=i+1:length(EV_M)
%         if abs(abs(EV_M(i))-abs(EV_M(j))) > eps 
%             error('Dilation matrix is NOT an isotropic matrix'); 
%         end 
%     end 
% end
rhoM = d; % the spectral radius of the dilation matrix M
if sumrule_k <= 0 error('The order of the sum rules must be at leat 1'); end

% Compute the set E_k
sumA0 = zeros(r,r);
for i=1:suppN
    sumA0 = sumA0 + mask_a(:,(i-1)*r+1:i*r);
end
EV_A = eig(sumA0);
if min(abs(EV_A-1)) > eps error('Mask a does NOT contaion 1 as an eigvalue!'); end
k = sumrule_k - 1;
E_k = [];
for i = 1:length(EV_A)
    lambda = EV_A(i);
    if abs(lambda-1) > eps
        for mu = 0:k 
            tmp = lambda*conj(dilation_d^(-mu));
            E_k = [E_k tmp conj(tmp)];
        end
    end
end
for mu = 0:2*k+1
    E_k=[E_k dilation_d^(-mu)];
end

%compute mask_b and supp_b
supp_b_tmp=[];
for i = 1:suppN
    for j = 1: suppN
        supp_b_tmp = union(supp_b_tmp, supp_a(i) - supp_a(j));
    end
end

supp_b = [];
mask_b = [];
len = size(supp_b_tmp,2);
for i = 1:len
    mask_b_i = zeros(r^2,r^2);
    for j = 1:suppN
        l = supp_a(j)+supp_b_tmp(i);
        [tf index] = ismember(l,supp_a);
        if tf
           mask_b_i=mask_b_i+kron(conj(mask_a(:,(j-1)*r+1:j*r)),mask_a(:,(index-1)*r+1:index*r));
        end
    end
    if norm(mask_b_i,inf) > eps
        supp_b = [supp_b supp_b_tmp(i)];
        mask_b = [mask_b mask_b_i*d];
    end
end

%compute set K = sum_{j=1}^{infinity}{dilation_d^{-j}*supp_b}
setK = [];
d1 = d - 1;
len = size(supp_b,2);
maxSupp_b = max(abs(supp_b));
N = ceil(maxSupp_b/d1);
K0 = [-N:N];
K1 = [];
while 1
    len1 = length(K0);
    len2 = length(supp_b);
    for i = 1:len1
        for j = 1:len2
            tmp = K0(i)+supp_b(j);
            if mod(tmp, d) == 0 
                K1 = union(K1,tmp/d);
            end
        end
    end
    tmp = intersect(K0,K1);
    if length(tmp) == length(K0)
        break;
    end
    K0 = K1;
    K1 = [];
end;
setK = K0;
len = size(setK,2);
matrixB = zeros(len*r^2,len*r^2);
for i = 1:len
    for j = 1:len
        l = dilation_d*setK(i)-setK(j);
        [tf index] = ismember(l,supp_b);
        if tf
            matrixB((i-1)*r^2+1:i*r^2, (j-1)*r^2+1:j*r^2) = mask_b(:,(index-1)*r^2+1:index*r^2);
        end
    end
end

EV_B = eig(matrixB);

for i = 1:length(E_k)
    for j = 1:length(EV_B)
        if abs(EV_B(j)-E_k(i)) < eps
            EV_B(j) = 0;
            break;
        end
    end
end
%C = setdiff(EV_B',E_k);
rho_k = norm(EV_B,inf);
smoothness = -log(sqrt(rho_k))/log(d);











