function [smoothness, rho_k, rhoM] = D2CriticalExponentV2(mask_a, supp_a, dilation_M, sumrule_k)
%#####################Copyright and Notice###################################
%Copyright since 2007, ZHUANG Xiaosheng, University of Alberta,
%                    All Rights Reserved 
%Report errors, mistakes, and bugs to ZHUANG Xiaosheng at xzhuang@math.ualberta.ca.
%Send comments/suggestions to ZHUANG Xiaosheng at xzhuang@math.ualberta.ca.
%
%  This program is based on the following paper (Theorem 7.1):
%  Bin Han, "Vector cascade algorithms and refinable function vectors in
%  Sobolev spaces", Journal of Approximation Theory, Volumn 124 (2003), 
%  Issue 1, 44-88. 
%
%  1. Refinement Equation(in Tex form)
%  \Phi(x) = [\Phi_1(x) ... \Phi_r(x)]^T
%  \Phi(x) = |detM|\sum_{\beta in Z^s}{a(\beta)*\Phi(Mx-\beta)} 
%
%  2. Parameters
%  dimension: s = 2
%  multiplicity: r integer
%  #{support Mask_A}=N: the size of the support of the mask.
%  Example: s=2, r=2, N=3
%
%
%  3. Input <=
%     3.1 mask_a is multi*(multi*N) matrix, can be seen as N blocks, each block is
%         a r*r matrix. Example:
%        mask_a = [ 1 1  2 2  3 3;
%                   1 1  2 2  3 3 ] has 3 blocks of 2*2 matrices.
%     3.2 supp_a is the support of the mask_a. It is an s*N matrix.
%         supp_a = [\beta_1 ... \beta_N]
%         \beta_1, .. , \beta_N are s*1 integer vectors 
%         Example: s=2, N=4
%         supp_a = [ -1 0 1 3]
%                  [ 0  1 2 2];

%         Note: mask_a(\beta_j) = the j_th block of Mask_A
%     3.3 dilation_M is the dialtion matrix which is isotropic.
%     3.4 sumrule_k. mask_a satisfies the sum rules of oder k with
%     respect to some vector \hat{y}(\xi).
%  4. Output =>
%     smoothness: the estimate of the L_2 critical exponent. see the
%     definition in the above paper.
% 
%  NOTE: Here we don't check the sum rule of the mask a. We assume that
%  mask a is obtained previously and already satisfied the sum rule of
%  order at least 1.
%
% Last modified:  July 10, 2006%

if nargin~=4 error('this routines requies 4 input argument'); end

%Check the arguments.
multi = size(mask_a,1); %multiplicity
s = size(supp_a,1); %dimension
numMaskSupp = size(supp_a,2);
detM =abs(det(dilation_M));
eps = 1e-7;
if numMaskSupp ~= size(mask_a,2)/multi error('Mask a and its support are not consistent!'); end

EV_M = eig(dilation_M);
for i=1:length(EV_M)-1
    for j=i+1:length(EV_M)
        if abs(abs(EV_M(i))-abs(EV_M(j))) > eps 
            error('Dilation matrix is NOT an isotropic matrix'); 
        end 
    end 
end
rhoM = abs(EV_M(1)); % the spectral radius of the dilation matrix M
if sumrule_k <= 0 error('The order of the sum rules must be at leat 1'); end

%compute mask_b and supp_b
supp_b=[];
mask_b=[];
for i = 1:numMaskSupp
    for j = 1: numMaskSupp
        flag = 0;
        tmpA = supp_a(:,i)-supp_a(:,j);
        if size(supp_b,2)>0
            tmp1 = supp_b(1,:) == tmpA(1);
            tmp2 = supp_b(2,:) == tmpA(2);
            tmp = tmp1.*tmp2;
            if max(tmp) == 1 
                continue;
            else
                flag = 1;
            end
        else
            flag = 1;
        end       
        tmp = zeros(multi^2,multi^2);
        for k = 1:numMaskSupp
            tmpk = supp_a(:,k)+tmpA;
            tmp1 = (supp_a(1,:) == tmpk(1));
            tmp2 = (supp_a(2,:) == tmpk(2));
            tmp3 = tmp1.*tmp2;
            [t,index]=ismember(1,tmp3);
            if t
               tmp=tmp+detM*kron(mask_a(:,(k-1)*multi+1:k*multi), mask_a(:,(index-1)*multi+1:index*multi));
            end
        end
        if norm(tmp)>0 && flag == 1
            supp_b = [supp_b, tmpA];
            mask_b = [mask_b, tmp];
        end 
    end
end
disp('done with computing mask_b, supp_b');
%compute set K =Z^s\cap sum_{j=1}^{infinity}{dilation_M^{-j}*supp_b}
h = max(max(abs(supp_b)));
r = ceil(sqrt(2)*rhoM*h/(rhoM-1));
K0 = zeros(2, (2*r+1)^2);
k = 1;
for i = -r:r
    for j = -r:r
        K0(:,k)=[i;j];
        k = k+1;
    end 
end 

a = dilation_M(1,1);
b = dilation_M(1,2);% M=[a b; c d]
c = dilation_M(2,1);% M^(-1)=1/(ad-bc)*[d -b; -c a];
d = dilation_M(2,2);% M^(-1)*[r1;r2] =  1/(ad-bc)*[d*r1-b*r2; -c*r1+a*r2];
while 1
    %compute K1 = M^(-1)*(K0+supp_b);
    K1 = [];
    len1 = size(K0,2);
    len2 = size(supp_b,2);
    for i = 1:len1
        for j = 1:len2
            tmpk = K0(:,i)+supp_b(:,j);
            a1 = d*tmpk(1)-b*tmpk(2);
            a2 = -c*tmpk(1)+a*tmpk(2);
            a3 = a*d-b*c; % M^(-1)*tmpk \in Z^2 ?
            if mod(a1,a3) == 0 && mod(a2,a3) == 0 
                if size(K1,2) > 0
                    tmp1 = (K1(1,:) == a1/a3);
                    tmp2 = (K1(2,:) == a2/a3);
                    tmp = tmp1.*tmp2;
                    if max(tmp) == 0
                       K1 = [K1, [a1/a3; a2/a3]];
                    end
                else
                    K1 = [K1, [a1/a3; a2/a3]];
                end 
            end
        end
    end
    % compute K2 = K1\cap K0
    K2 = [];
    for i = 1: size(K1,2)
        tmp1 = (K0(1,:) == K1(1,i));
        tmp2 = (K0(2,:) == K1(2,i));
        tmp = tmp1.*tmp2;
        if max(tmp) == 1
            K2 = [K2, K1(:,i)];
        end
    end 
    if size(K0,2) == size(K2,2) 
        break;
    end  
    K0 = K2;
end;
[tmp index] = sort(K0(1,:));
setK= K0(:,index);
%setK = K0;
disp('done with computing set K');
% Compute the set E_k
sumA0 = zeros(multi,multi);
for i=1:numMaskSupp
    sumA0 = sumA0 + mask_a(:,(i-1)*multi+1:i*multi);
end
EV_A = eig(sumA0);
if min(abs(EV_A-1)) > eps error('Mask a does NOT contaion 1 as an eigvalue!'); end
k = sumrule_k - 1;
E_k = []; sigma1 = EV_M(1); sigma2 = EV_M(2);
for i = 1:length(EV_A)
    lambda = EV_A(i);
    if abs(lambda-1) > eps
        mu1 = 0; mu2 = 0; total = 0;
        for j = 1: (k+1)*(k+2)/2
            tmp = lambda*conj(sigma1^(-mu1)*sigma2^(-mu2));
            E_k = [E_k, tmp, conj(tmp)];
            if mu1 < total 
                mu1 = mu1+1; mu2 = mu2-1;
            else
                total = total+1; mu1 = 0; mu2 = total;
            end
        end
    end
end

mu1 = 0; mu2 = 0; total = 0;
for i = 1:(2*k+2)*(2*k+3)/2
    tmp = sigma1^(-mu1)*sigma2^(-mu2);
    E_k = [E_k, tmp];
    if mu1 < total 
        mu1 = mu1+1; mu2 = mu2 -1;
    else
        total = total + 1; mu1 =0; mu2 = total;
    end
end
E_k=sort(E_k);
disp('done with computing set E_k');
% Now compute \rho_k(a,M,p,y)
len = size(setK,2);
matrixB = zeros(len*multi^2,len*multi^2);
for i = 1:len
    for j = 1:len
        tmpK = dilation_M*setK(:,i)-setK(:,j);
        tmp1 = (supp_b(1,:) == tmpK(1));
        tmp2 = (supp_b(2,:) == tmpK(2));
        tmp = tmp1.*tmp2;
        [tf index] = ismember(1,tmp);
        if tf
             matrixB((i-1)*multi^2+1:i*multi^2, (j-1)*multi^2+1:j*multi^2) = mask_b(:,(index-1)*multi^2+1:index*multi^2);
        end
    end
end

EV_B = eig(matrixB);

for i = 1:length(E_k)
    for j = 1:length(EV_B)
        if abs(EV_B(j)-E_k(i)) < eps
            EV_B(j) = 0;
            break;
        end
    end
end
disp('done with computing spec(b(Ma-b))\E_k');
%C = setdiff(EV_B',E_k);
rho_k = norm(EV_B,inf);
smoothness = -log(sqrt(rho_k))/log(rhoM);











